local mod = ...



-- Job: Flax Farmer -------------------------------------------------------------------------------------
mod:register({
	DataType = "JOB",
	Id = "JOB_FLAXFARMER",
	JobName = "JOB_FLAXFARMER_NAME",
	JobDescription = "JOB_FLAXFARMER_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_FARMER",
	UseWorkplaceBehavior = true,
    ProductionDelay = 0.1, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP", 
		-- LeftHandObject = {
		-- 	Model = "PREFAB_TOOL_FARMER_SEED_BAG", 
		-- 	WorkAttachName = "SeedBag_Anchor", 
		-- 	WalkAttachName = "SeedBag_Anchor"
		-- },
		WalkAnimation = "WALKING",
		IdleAnimation = "IDLE"
	}
})

mod:override({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_FLAXFARMER"
    }
})

mod:override({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_FLAXFARMER"
    }
})

mod:override({
    Id = "COMMONER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_FLAXFARMER"
    }
})



-- Job: Flax Worker -------------------------------------------------------------------------------------
mod:register({
	DataType = "JOB",
	Id = "JOB_FLAXWORKER",
	JobName = "JOB_FLAXWORKER_NAME",
	JobDescription = "JOB_FLAXWORKER_DESC",
	IsLockedByDefault = false,
	DefaultBehavior = "BEHAVIOR_WORK",
	-- UseWorkplaceBehavior = true,
    ProductionDelay = 20.0, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP", 
		-- LeftHandObject = {
		-- 	Model = "PREFAB_TOOL_FARMER_SEED_BAG", 
		-- 	WorkAttachName = "SeedBag_Anchor", 
		-- 	WalkAttachName = "SeedBag_Anchor"
		-- },
		WalkAnimation = "WALKING",
		IdleAnimation = "IDLE"
	}
})

mod:override({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_FLAXWORKER"
    }
})

mod:override({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_FLAXWORKER"
    }
})

mod:override({
    Id = "COMMONER",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_FLAXWORKER"
    }
})



-- Resource -------------------------------------------------------------------------------------
mod:registerAssetId("icons/FlaxGreen.png", "ICON_FLAX_GREEN")

mod:register({
	DataType = "RESOURCE",
	Id = "RESOURCE_FLAX_GREEN",
	ResourceName = "RESOURCE_FLAX_GREEN_NAME",
	Icon = "ICON_FLAX_GREEN",
	ResourceTypeList = { "RESOURCE" },
	IsTradable = false,
	ResourceLayoutType = "CRATES",
	-- VillagerBuyingPrice = { Resource = "GOLD", Quantity = 10 },
	DisplayInInventory = false
})


mod:registerAssetId("icons/FlaxRetted.png", "ICON_FLAX_RETTED")

mod:register({
	DataType = "RESOURCE",
	Id = "RESOURCE_FLAX_RETTED",
	ResourceName = "RESOURCE_FLAX_RETTED_NAME",
	Icon = "ICON_FLAX_RETTED",
	ResourceTypeList = { "RESOURCE" },
	IsTradable = false,
	ResourceLayoutType = "CRATES",
	-- VillagerBuyingPrice = { Resource = "GOLD", Quantity = 10 },
	DisplayInInventory = false
})


-- Moved to Common Resources
-- mod:registerAssetId("icons/Flax.png", "ICON_FLAX")

-- mod:register({
-- 	DataType = "RESOURCE",
-- 	Id = "RESOURCE_FLAX",
-- 	ResourceName = "RESOURCE_FLAX_NAME",
-- 	Icon = "ICON_FLAX",
-- 	ResourceTypeList = { "RESOURCE" },
-- 	IsTradable = true,
-- 	ResourceLayoutType = "CRATES",
-- 	-- VillagerBuyingPrice = { Resource = "GOLD", Quantity = 10 },
-- 	DisplayInInventory = true
-- })



-- Assets
mod:registerAssetId("models/Flax1.fbx/Prefab/FarmPart", "PREFAB_BUILDING_PART_FLAXFARM")
mod:registerAssetId("models/Flax1.fbx/Prefab/PondPart", "PREFAB_BUILDING_PART_FLAXPOND")
mod:registerAssetId("models/Flax1.fbx/Prefab/YardPart", "PREFAB_BUILDING_PART_FLAXYARD")

-- Models used for the plants must have only a single Mesh and a single Material, otherwise planting will crash!
mod:registerAssetId("models/Flax3.fbx/Prefab/Flax", "PREFAB_PLANT_FLAX")

mod:registerAssetId("models/Textures/Colors.png", "TEXTURE_FLAX")




-- Building Asset Processor
mod:registerAssetProcessor("models/Flax1.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Construction visual
mod:registerAssetId("models/Flax1.fbx/Prefab/FarmConstruction", "PREFAB_CONSTR_BUILDING_PART_FLAXFARM")
mod:registerAssetId("models/Flax1.fbx/Prefab/PondConstruction", "PREFAB_CONSTR_BUILDING_PART_FLAXPOND")
mod:registerAssetId("models/Flax1.fbx/Prefab/YardConstruction", "PREFAB_CONSTR_BUILDING_PART_FLAXYARD")



-- Collider Platform (climbable mesh)  -------------------------------------------------------------------------------------
-- mod:configurePrefabFlagList("models/JoinerWorkshop1.fbx/Prefab/WorkshopPart", { "PLATFORM" })



-- Grounding
for i=1, 3, 1 do
	mod:registerPrefabComponent("models/Flax1.fbx/Prefab/YardPart/Sheaf." .. string.sub("000" .. i, -3), {DataType = "COMP_GROUNDED"})
end



-- Building part type
mod:registerEnumValue("BUILDING_PART_TYPE", "FARM")
mod:registerEnumValue("BUILDING_PART_TYPE", "POND")
mod:registerEnumValue("BUILDING_PART_TYPE", "YARD")



-- Alpha material (transparency) -------------------------------------------------------------------------------------
mod:registerAssetId("models/Flax3.fbx/Materials/Material.Flax", "MATERIAL_FLAX")

mod:override({
    Id = "MATERIAL_FLAX",
    HasAlphaTest = true
})



-- Crop Field Element -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Flax3.fbx/Prefab/Flax", { DataType = "COMP_CROP_FIELD_ELEMENT" })



-- Plantable -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/Flax3.fbx/Prefab/Flax", { DataType = "COMP_PLANTABLE", MinSpaceBetweenPlant = 0.2 })

mod:register({
    DataType = "PLANTABLE",
    Id = "PLANTABLE_FLAX",
    Name = "PLANTABLE_FLAX_NAME",
    PlantPrefab = "PREFAB_PLANT_FLAX",
    MinimumScale = 0.85, -- min size at full growth
    MaximumScale = 1.15 -- max size at full growth
})



-- Resource Container -------------------------------------------------------------------------------------
mod:registerPrefabComponent( "models/Flax3.fbx/Prefab/Flax", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "RESOURCE_FLAX_GREEN",
    IsReplenishable = false,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 2.00, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = true,
    HasMaximumWorkstation = false
})



-- Zone -------------------------------------------------------------------------------------
mod:register({
    DataType = "ZONE",
    Id = "ZONE_FLAXFARM",
    Name = "ZONE_FLAXFARM_NAME",
    Description = "ZONE_FLAXFARM_DESC",
    Color = {1.00, 1.70, 0.60, 0.9}, -- RGB A = 
    DesirabilityLayerToShow = "DESIRABILITY_FERTILITY"
})



-- Farm Field Config -------------------------------------------------------------------------------------
mod:register({
    DataType = "FARM_FIELD_CONFIG",
    Id = "FARM_FIELD_CONFIG_FLAXFARM",
    ZoneType = "ZONE_FLAXFARM",
    PlantPrefab = "PREFAB_PLANT_FLAX",
    HasPlantingCycle = true,
    IsScaling = true,
    RowDistance = 0.5,
    ItemDistance = 0.5,
    RandomOrientation = {0, 360},
    -- RandomPositionOffset = {0, 0},
    RandomPositionOffset = {-0.2, 0.2},
    SourceMaterial = "MATERIAL_FLAX",
    GrowingMaterial = "MATERIAL_FLAX",
    HarvestedMaterial = "MATERIAL_FLAX",
    GrowingTextureList = {
        "TEXTURE_FLAX",
    },
    HarvestingTextureList = {
        "TEXTURE_FLAX",
    },
    AlphaTestThreshold = 0.5
})



-- Growing Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_FLAXFARM_GROWING",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
        -- LeftHandObject = {
        --     Model = "PREFAB_TOOL_FARMER_SEED_BAG", 
        --     WorkAttachName = "SeedBag_Anchor", 
		-- 	WalkAttachName = "SeedBag_Anchor",
		-- 	KeepObjectOnWalkCycle = false
        -- },
        -- FemaleHatList = {
        --     "PREFAB_HAT_FARMER3",
        -- }, 
        -- MaleHatList = {
        --     "PREFAB_HAT_FARMER3",
        -- }, 
        WalkAnimation = "WALKING",
        IdleAnimation = "GRAPE_CARING"
	}
})



-- Sowing Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_FLAXFARM_SOWING",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
        LeftHandObject = {
            Model = "PREFAB_TOOL_FARMER_SEED_BAG", 
            WorkAttachName = "SeedBag_Anchor", 
			WalkAttachName = "SeedBag_Anchor",
			KeepObjectOnWalkCycle = false
        },
        -- FemaleHatList = {
        --     "PREFAB_HAT_FARMER3",
        -- }, 
        -- MaleHatList = {
        --     "PREFAB_HAT_FARMER3",
        -- }, 
        WalkAnimation = "SEED_WALK",
        IdleAnimation = "SEED"
	}
})



-- Harvest Setup -------------------------------------------------------------------------------------

mod:register({
    DataType = "AGENT_WORKSTATION_SETUP",
    Id = "AGENT_WORKSTATION_SETUP_FLAXFARM_HARVEST",
	CharacterSetup = {
        DataType = "CHARACTER_SETUP",
        RightHandObject = {
			Model = "PREFAB_TOOL_FARMER_SICKLE", 
			KeepObjectOnWalkCycle = true,
            WorkAttachName = "Sickle_Anchor", 
			WalkAttachName = "Sickle_Anchor",
			KeepObjectOnWalkCycle = false
        },
        -- FemaleHatList = {
        --     "PREFAB_HAT_FARMER1",
        -- }, 
        -- MaleHatList = {
        --     "PREFAB_HAT_FARMER2",
        -- }, 
        WalkAnimation = "SCYTHE_WALK",
		IdleAnimation = "SCYTHE"
	}
})



-- Building Function: Flax Farm -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_FARM",
    Id = "BUILDING_FUNCTION_FLAXFARM",
    Name = "BUILDING_FUNCTION_FLAXFARM_NAME",
    WorkerCapacity = 3,
    RelatedJob = { 
        Job = "JOB_FLAXFARMER", 
        Behavior = "BEHAVIOR_FARMER"
    },
    -- InputInventoryCapacity = {
	-- 	{ Resource = "WATER", Quantity = 5 },
	-- 	-- { Resource = "DUNG_RESOURCE", Quantity = 5 }
	-- },
    ResourceListNeeded = {
		{ Resource = "RESOURCE_FLAX_GREEN", Quantity = 1 }, 
	},
	ResourceProduced = {
        { Resource = "RESOURCE_FLAX_GREEN", Quantity = 1 },
    },

    -- there is a max total cycle of 21 days so if you plan beyond that, it will be cut short
    SowingDurationPerc = {0, 6},
	GrowDuration = {6, 10},
	GrowWaitDuration = {10, 12},
	HarvestDuration = {12, 18},
	RestDuration = {18, 21},

    QuantityToHarvestPerStep = 30,
	HarvestStepsCount = 20,
	QuantityToPlantPerStep = 25,
	PlantingStepsCount = 20,
	ResourceValuePerGather = 0.02,

    CropFieldConfig = "FARM_FIELD_CONFIG_FLAXFARM",
    GrowingSetup = "AGENT_WORKSTATION_SETUP_FLAXFARM_GROWING", 
    SowingSetup = "AGENT_WORKSTATION_SETUP_FLAXFARM_SOWING", 
    HarvestSetup = "AGENT_WORKSTATION_SETUP_FLAXFARM_HARVEST"
})



-- Building Function: Flax Retting Pond  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_FLAXPOND",
    Name = "BUILDING_FUNCTION_FLAXPOND_NAME",
    WorkerCapacity = 1,
    RelatedJob = { 
        Job = "JOB_FLAXWORKER", 
        Behavior = "BEHAVIOR_WORK"
    },
    InputInventoryCapacity = {
		{ Resource = "RESOURCE_FLAX_GREEN", Quantity = 25 },
	},
    ResourceListNeeded = {
		{ Resource = "RESOURCE_FLAX_GREEN", Quantity = 5 }
	},
    ResourceProduced = {
		{ Resource = "RESOURCE_FLAX_RETTED", Quantity = 5 }
	}
})



-- Building Function: Flax Crushing and Combing Yard  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_FLAXYARD",
    Name = "BUILDING_FUNCTION_FLAXYARD_NAME",
    WorkerCapacity = 1,
    RelatedJob = { 
        Job = "JOB_FLAXWORKER", 
        Behavior = "BEHAVIOR_WORK"
    },
    InputInventoryCapacity = {
		{ Resource = "RESOURCE_FLAX_RETTED", Quantity = 25 },
	},
    ResourceListNeeded = {
		{ Resource = "RESOURCE_FLAX_RETTED", Quantity = 5 }
	},
    ResourceProduced = {
		{ Resource = "RESOURCE_FLAX", Quantity = 5 }
	}
})



-- Building Function: Weaver Flax  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_WEAVER_FLAX",
    Name = "BUILDING_FUNCTION_WEAVER_FLAX_NAME",
    WorkerCapacity = 2,
    RelatedJob = { 
        Job = "WEAVER", 
        Behavior = "BEHAVIOR_WORK"
    },
    InputInventoryCapacity = {
		{ Resource = "RESOURCE_FLAX", Quantity = 50 },
	},
    ResourceListNeeded = {
		{ Resource = "RESOURCE_FLAX", Quantity = 1 }
	},
    ResourceProduced = {
		{ Resource = "WOOLEN_CLOTH", Quantity = 1 }
	}
})



-- Building Function Assignable  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_ASSIGNABLE",
    Id = "BUILDING_FUNCTION_ASSIGNABLE_WEAVER",
	Name = "BUILDING_FUNCTION_ASSIGNABLE_WEAVER_NAME",
	AuthorizedFunctionList = "BUILDING_FUNCTION_LIST_WEAVER"
})



-- Building Function List  -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_LIST",
    Id = "BUILDING_FUNCTION_LIST_WEAVER",
	AssetFunctionList = {
		"BUILDING_FUNCTION_WEAVER_HUT",
		"BUILDING_FUNCTION_WEAVER_FLAX",
	}
})



-- Builing Part Weaver Hut (override) -------------------------------------------------------------------------------------

mod:override({
	Id = "BUILDING_PART_WEAVER_HUT",
    AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_WEAVER"
})

mod:override({
	Id = "BUILDING_FUNCTION_WEAVER_HUT",
    Name = "BUILDING_FUNCTION_WEAVER_HUT_NAME"
})



-- Building: Flax Farm -------------------------------------------------------------------------------------

mod:register({
	DataType = "BUILDING", 
	Id = "BUILDING_FLAXFARM",
	Name = "BUILDING_FLAXFARM_NAME",
	Description = "BUILDING_FLAXFARM_DESC",
	BuildingType = "GENERAL",
	VillagerRequired = { Status = "SERF", Quantity = 25 },
	AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
	BuildingPartSetList = {
		{
			Name = "BUILDING_SET_FLAXFARM1_NAME",
			BuildingPartList = { 
				"BUILDING_PART_FLAXFARM",
			}
		},
		{
			Name = "BUILDING_SET_FLAXFARM2_NAME",
			BuildingPartList = { 
				"BUILDING_PART_FLAXPOND",
			}
		},
		{
			Name = "BUILDING_SET_FLAXFARM3_NAME",
			BuildingPartList = { 
				"BUILDING_PART_FLAXYARD",
			}
		},
	},	
	RequiredPartList = {
		-- { Category = "CORE", Min = 1, Max = 1 }
	},
	IsEditable = true	
})



-- Building Part: Farm -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_FLAXFARM",
	Name = "BUILDING_PART_FLAXFARM_NAME", 
	Description = "BUILDING_PART_FLAXFARM_DESC", 
	Category = "FARM", 
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_BUILDING_PART_FLAXFARM"
	},
	ConstructionVisual = "PREFAB_CONSTR_BUILDING_PART_FLAXFARM",
	BuildingZone = { 
        ZoneEntryList = {
            { 
                Polygon = polygon.createRectangle( {7.0, 5.0}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false } 
			},
            { 
                Polygon = polygon.createRectangle( {6.0, 4.0}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
			},
		}
	},
	AssetBuildingFunction = "BUILDING_FUNCTION_FLAXFARM",
	Cost = {
		BuildRightTaxes = {	{ Resource = "GOLD", Quantity = 5 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 2 } },
		RessourcesNeeded = {
            { Resource = "WOOD", Quantity = 10 },
            { Resource = "TOOL", Quantity = 3 }
		}
	}
})



-- Building Part: Retting Pond -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_FLAXPOND",
	Name = "BUILDING_PART_FLAXPOND_NAME", 
	Description = "BUILDING_PART_FLAXPOND_DESC", 
	Category = "POND", 
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_BUILDING_PART_FLAXPOND"
	},
	ConstructionVisual = "PREFAB_CONSTR_BUILDING_PART_FLAXPOND",
	BuildingZone = { 
        ZoneEntryList = {
            { 
                Polygon = polygon.createCircle( 3.25, {0.0, 0.0}, 12 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false } 
			},
            { 
                Polygon = polygon.createCircle( 2.25, {0.0, 0.0}, 12 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
			},
		}
	},
	AssetBuildingFunction = "BUILDING_FUNCTION_FLAXPOND",
	Cost = {
		BuildRightTaxes = {	{ Resource = "GOLD", Quantity = 60 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 5 } },
		RessourcesNeeded = {
            { Resource = "WOOD", Quantity = 14 },
		}
	}
})



-- Building Part: Combing and Crushing Yard -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_FLAXYARD",
	Name = "BUILDING_PART_FLAXYARD_NAME", 
	Description = "BUILDING_PART_FLAXYARD_DESC", 
	Category = "YARD", 
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_BUILDING_PART_FLAXYARD"
	},
	ConstructionVisual = "PREFAB_CONSTR_BUILDING_PART_FLAXYARD",
	BuildingZone = { 
        ZoneEntryList = {
            { 
                Polygon = polygon.createRectangle( {4.0, 5.2}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false } 
			},
            { 
                Polygon = polygon.createRectangle( {3.5, 4.5}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
			},
		}
	},
	AssetBuildingFunction = "BUILDING_FUNCTION_FLAXYARD",
	Cost = {
		BuildRightTaxes = {	{ Resource = "GOLD", Quantity = 5 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 1 } },
		RessourcesNeeded = {
            { Resource = "WOOD", Quantity = 4 },
		}
	}
})

